class CircularReferenceInternal(ValueError):
	def __init__(self, msg, node):
		super(CircularReferenceInternal, self).__init__(self, msg)
		self.node = node
class CircularReferenceError(ValueError):
	pass

class DepGraph(object):
	def __init__(self):
		self.graph = []
		self.reference_map = {}
		self.resolving_map = {}
		self.resolved_map = {}
		self.resolution = []
		self.resolving = []

	def add(self, ident, deps):
		dgn = (ident, deps)
		self.graph.append(dgn)
		self.reference_map[ident] = dgn

	def resolve_node(self, node):
		self.resolving.append(node[0])
		if node[0] in self.resolving_map:
			raise CircularReferenceInternal("Circular reference", node[0])

		if node[0] in self.resolved_map:
			#print("already resolved")
			return

		self.resolving_map[node[0]] = 1
		if node[1]:
			for dep in node[1]:
				self.resolve_node(self.reference_map[dep])
		self.resolution.append(node[0])
		self.resolved_map[node[0]] = node
			


	def resolve(self):
		self.resolved_map = {}
		self.resolution = []
		for g in self.graph:
			self.resolving_map = {}
			self.resolving = []
			err = None
			try:
				self.resolve_node(g)
			except CircularReferenceInternal:
				circ = []
				for v in reversed(self.resolving):
					if v in self.resolved_map:
						continue
					if v == self.resolving[-1]:
						break
					circ.append(v)

				err = CircularReferenceError("Circular reference detected between %s" % (tuple(circ),))

			if err:
				raise err


	def bottomup_iter(self):
		return self.resolution








if __name__ == "__main__":

	dg = DepGraph()
	dg.add("a", ["d", "c"])
	dg.add("b", ["c"])
	dg.add("d", [])
	dg.add("c", [])
	dg.resolve()
	print(list(dg.bottomup_iter()))

	dg = DepGraph()
	dg.add("a", ["d", "c"])
	dg.add("b", ["c"])
	dg.add("d", [])
	dg.add("e", [])
	dg.add("f", ["e"])
	dg.add("ko", ["f"])
	dg.add("j", [])
	dg.add("d", [])
	dg.add("c", [])
	dg.resolve()
	print(list(dg.bottomup_iter()))
	
	try:
		dg = DepGraph()
		dg.add("a", ["d", "c"])
		dg.add("b", ["c"])
		dg.add("d", [])
		dg.add("c", ["j"])
		dg.resolve()
		print(list(dg.bottomup_iter()))
	except:
		print("error (as expected)")

	dg = DepGraph()
	dg.add("a", ["d", "c"])
	dg.add("b", ["c"])
	dg.add("d", [])
	dg.add("c", ["d"])
	dg.resolve()
	print(list(dg.bottomup_iter()))


